#!/usr/bin/env python3

"""
Child class of JournalFetcher
Usage: Check if Url can be used with 'can_use_url'
       and then fetch publication with 'get_publication'
"""

import re

from input.get.journal_fetcher import JournalFetcher
from input.publication import Publication, Citation


class Fetcher(JournalFetcher):
    """
    Specific Fetcher for the ACS journals.
    """

    # Constant for the abbreviations of the supported Journals
    SUPPORTED_JOURNALS = ['1021']

    @staticmethod
    def can_use_url(url: str) -> bool:
        """
        Uses Regex to extract journal specific substrings in Doi.
        TODO: Support non Doi-urls
        """
        matched_url = re.match(r'(https?://)?(doi.org/)?(10.(\d{4})/\w+.\S+)', url)
        return matched_url[4] in Fetcher.SUPPORTED_JOURNALS

    @staticmethod
    def get_publication(url: str) -> Publication:
        """
        Fetches html and creates Beatifulsoup-instance in parent class.
        Specific css-searches for ACS-Journals and creates Publication-instance.
        """

        # Creation of Soup
        soup = JournalFetcher.get_soup(url)
        soup_header = soup.select('.article_header')[0]
        ref_cit_soup = soup

        # Creates Publication
        doi_url = soup_header.select('a[title="DOI URL"]')[0].string
        title = soup_header.select(".hlFld-Title")[0].text

        contributors = []
        for author in soup_header.select(".hlFld-ContribAuthor"):
            contributors.append(author.text)

        journal = soup_header.select(".cit-title")[0].text

        published = soup_header.select(".pub-date-value")[0].text

        subjects = []
        subject_soup = soup_header.select('.article_header-taxonomy')[0]
        for subject in subject_soup.select('a'):
            subjects.append(subject.text)

        num_citations = 0


        references = []
        references_soup = ref_cit_soup.select('ol#references')
        if references_soup != []:
            for reference in references_soup[0].select('li'):
                ref_doi = "https://doi.org/{}".format(reference.select('.refDoi')[0].text.strip()[5:])\
                        if reference.select('.refDoi') != [] else "None"
                ref_title = reference.select('.NLM_article-title')[0].text\
                        if reference.select('.NLM_article-title') != [] else "None"
                ref_journal = reference.select('i')[0].text\
                        if reference.select('i') != [] else "None"
                
                ref_contributors=[]
                for author in reference.select('.NLM_contrib-group'):
                    ref_contributors.append(author.text)

                references.append(Citation(ref_doi, ref_title, ref_journal, ref_contributors, cit_type="Reference"))

        citations = []
        citation_soup = ref_cit_soup.select('.cited-content_cbyCitation')
        if citation_soup != []:
            for citation in citation_soup[0].select('li'):
                cit_doi = citation.select('a[title="DOI URL"]')[0].text\
                        if citation.select('a[title="DOI URL"]') != [] else "None"
                cit_title = citation.select('.cited-content_cbyCitation_article-title')[0].text\
                        if citation.select('.cited-content_cbyCitation_article-title')!= [] else "None"
                cit_journal = citation.select('.cited-content_cbyCitation_journal-name')[0].text\
                        if citation.select('.cited-content_cbyCitation_journal-name') != [] else "None"
                cit_contributors =[]
                cit_contributors = citation.select('.cited-content_cbyCitation_article-contributors')[0].text.split(', ')
        #           clean up of the last Entry
                cit_contributors_last = cit_contributors.pop().strip(". ")
                if cit_contributors_last != '':
                    cit_contributors.append(cit_contributors_last)  
                citations.append(Citation(cit_doi, cit_title, cit_journal, cit_contributors, cit_type = "Citation"))

        return Publication(doi_url, title, contributors, journal, published
                            , subjects, num_citations, references, citations)


    @staticmethod
    def test_fetcher():
        pass
