#!/usr/bin/env python3

"""
Interface for the Input-Package only this should be accessed from outside this Package.

"""
from os import walk
import importlib
import pathlib
import re
from input.publication import Publication

class InputInterface:
    """
    Singleton which dynamically imports and manages fetchers
    """

    get_path = None
    fetcher_classes=[]

    @staticmethod
    def get_publication(url: str) -> Publication:
        """
        The interface-method to get a Publication-instance

        Parameters
        ----------
        :param url: url to a Publication
        :type url: str
        :return: Publication instance or None if not supported
        """
        # Initializes 'fetcher_classes', the list of imported modules
        if InputInterface.fetcher_classes ==[]:
            InputInterface.get_fetcher_classes()
            if InputInterface.fetcher_classes ==[]:
                raise AttributeError("No specific Fetchers where found at: '{}'"
                                    .format(InputInterface.get_path))
        
        # Checks if module supports the 'url' and returns a Publication if it does.
        for fetcher_class in InputInterface.fetcher_classes:
            if fetcher_class.can_use_url(url):
                return fetcher_class.get_publication(url)
        
        # No Module for given url was found
        return None
        

    @staticmethod
    def get_fetcher_classes():
        """
        Searches in 'get', if there are [a-z]*.py modules (specific Fetchers)
        and tries to import them.
        Saves found modules in 'fetcher_files'.
        """

        # Path to 'get'-package
        InputInterface.get_path = '{}/get'.format(pathlib.Path(__file__).parent.resolve())
        
        # Searches for modules with given Pattern
        fetcher_file_names=[]
        for file in next(walk(InputInterface.get_path), (None, None, []))[2]:
            if re.match(r'[a-z]+.py', file) is not None:
                fetcher_file_names.append(file)

        if fetcher_file_names !=[]:
            print("Found following Modules: {}".format(", ".join(fetcher_file_names)))

        # Tries to import those modules and saves their 'Fetcher'-class
        for file in fetcher_file_names:
            try:
                fetcher_class = importlib.import_module("input.get.{}".format(file[:-3]))
                try:
                    InputInterface.fetcher_classes.append(fetcher_class.__getattribute__('Fetcher'))
                except Exception as error:
                    print("Module '{}' does not have a 'Fetcher'-class".format(file[:-3]))
            except Exception:
                raise ImportError("Module '{}' can not be imported".format(file[:-3]))
